package org.once.serviceChoicer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.once.serviceChoicer.candidate.ICandidate;
import org.once.serviceChoicer.candidate.IUDDI;
import org.once.serviceChoicer.exception.ServiceChoicerException;
import org.once.serviceChoicer.input.process.IEPath;
import org.once.serviceChoicer.input.process.IProcess;
import org.once.serviceChoicer.input.process.IService;
import org.once.serviceChoicer.input.qos.IQoSAttrs;
import org.once.serviceChoicer.input.qos.IQoSAttr.Type;
import org.once.serviceChoicer.input.qos.impl.QoSAttrs;
import org.once.serviceChoicer.lingo.ForLingo;
import org.once.serviceChoicer.math.IPolynomial;
import org.once.serviceChoicer.math.IVariable;
import org.once.serviceChoicer.math.impl.Polynomial;
import org.once.serviceChoicer.output.IConstrain;
import org.once.serviceChoicer.output.ITarget;
import org.once.serviceChoicer.output.IVarManager;
import org.once.serviceChoicer.output.IConstrain.OPType;
import org.once.serviceChoicer.output.impl.LinerConstrain;
import org.once.serviceChoicer.output.impl.QoSConstrain;
import org.once.serviceChoicer.output.impl.Target;
import org.once.serviceChoicer.output.impl.VarManager;
import org.once.serviceChoicer.system.IAggrCaculator;
import org.once.serviceChoicer.system.ICaculModle;
import org.once.serviceChoicer.system.impl.AggrCaculator;
import org.once.serviceChoicer.system.impl.CompServiceQoS;
import org.once.serviceChoicer.system.impl.EPathQoS;
import org.once.serviceChoicer.system.impl.FScore;
import org.once.serviceChoicer.system.impl.Liner;
import org.once.serviceChoicer.system.util.AggrTable;
import org.once.serviceChoicer.system.util.IAggrTable;

public class ServiceChoicer {
	// input
	private IProcess process;
	private IUDDI uddi;
	private IQoSAttrs qosAttrs;
	
	// manager
	private IAggrTable aggrTable;
	private IVarManager varManager;
	
	// 
	private double obj;
	private List<Double> results;
	
	
	// output
	private ITarget target;
	private List<IConstrain> subjects;
	
	private List<IVariable> outputVars;
	private Map<IVariable, String> var_CandNameMap;
	private Map<IVariable, ICandidate> var_NodeIDMap; 
	
	/*
	 *  constructor
	 */
	public ServiceChoicer(IProcess process, IQoSAttrs qosAttrs, IUDDI uddi) throws Exception{
		
		this.process = process;
		this.uddi = uddi;
		this.qosAttrs = new QoSAttrs();
		for(Type type : qosAttrs.getAttrTypes()){
			if(qosAttrs.getWeight(type)!=0)
			{
				this.qosAttrs.addQoSAttr(qosAttrs.get(type));
			}
		}
		
		init();
	}
	
	private void init(){
		aggrTable = new AggrTable();
		varManager = new VarManager();
	}
	
	private void createOutputVars() throws Exception {
		outputVars = new ArrayList<IVariable>();
		var_CandNameMap = new HashMap<IVariable, String>();
		var_NodeIDMap = new HashMap<IVariable,ICandidate>();
		
		for(Type type : qosAttrs.getAttrTypes()){
			outputVars.add(varManager.getVar(type, true));
		}
		for(Type type : qosAttrs.getAttrTypes()){
			outputVars.add(varManager.getVar(type, false));
		}
		for(IService service : process.getServices()){
			List<ICandidate> list = uddi.getCandidates(service);
			for(ICandidate cand : list){
				IVariable var = varManager.getVar(cand);
				outputVars.add(var);
				var_CandNameMap.put(var, cand.getName());
				var_NodeIDMap.put(var, cand.clone());
			}
		}
		
	}

	/**
	 * create the target function " max ......"u
	 * @return 
	 * @throws ServiceChoicerException
	 * Լ۸ԵļȨƽ 
	 */
	private void createTarget() throws ServiceChoicerException{	
		ICaculModle comp = new CompServiceQoS(qosAttrs, varManager);
		IPolynomial pn = comp.getPNomial();
		target = new Target(ITarget.Type.MAX, pn); 
	}
	
	/**
	 * create scoreModel
	 * @param list
	 * @throws ServiceChoicerException
	 * Ե
	 */
	private void addScoreModel(List<IConstrain> list) throws ServiceChoicerException{
		ICaculModle fscore = new FScore(qosAttrs,varManager,aggrTable);
		for(Type type : qosAttrs.getAttrTypes()){
			IVariable var = varManager.getVar(type, true);
			((FScore)fscore).setType(type);
			IPolynomial pn = fscore.getPNomial();
			IConstrain cons = new QoSConstrain(var, OPType.EQUAL, pn);
			list.add(cons);
		}
	}
	/**
	 * create epaht model
	 * @param list
	 * @throws ServiceChoicerException
	 * ÿEPathĸʺϳļ۸ֵ
	 */
	private void addEPathModel(List<IConstrain> list) throws ServiceChoicerException{
		ICaculModle epathQoS = new EPathQoS(process, varManager);
		for(Type type : qosAttrs.getAttrTypes()){
			IVariable var = varManager.getVar(type, false);
			((EPathQoS)epathQoS).setType(type);
			IPolynomial pn = epathQoS.getPNomial(); 
			IConstrain cons = new QoSConstrain(var, OPType.EQUAL, pn);
			list.add(cons);
		}
	}
	
	/**
	 * create aggragetion model
	 * @param list
	 * @throws Exception
	 */
	private void addArrgModel(List<IConstrain> list) throws Exception{
		IAggrCaculator aggrCaculator = new AggrCaculator(uddi, varManager, aggrTable);	
		for(IEPath ePath : process.getEPathes()){
			for(Type type : qosAttrs.getAttrTypes()){
				list.addAll(aggrCaculator.getConstrains(ePath,type)); 
			}
		}
	}
	
	/**
	 * create default subjects model
	 * @param list
	 * @throws Exception
	 */
	private void addDefSubjects(List<IConstrain> list) throws Exception{
		List<IService> sList = this.process.getServices();
		for(int i = 0; i < sList.size(); i++){
			List<ICandidate> cList = this.uddi.getCandidates(sList.get(i));
			IPolynomial temp;
			if(cList.size()>0){
				temp = new Polynomial(this.varManager.getVar(cList.get(0)));
				for(int j = 1; j < cList.size(); j++){
					temp.add(new Polynomial(this.varManager.getVar(cList.get(j))));
				}
				list.add(new LinerConstrain(temp, OPType.EQUAL, 1));
			}
		}
	}
	
	private void createSubjects() throws Exception{
		subjects = new ArrayList<IConstrain>();
		addScoreModel(subjects);
		addEPathModel(subjects);		
		addArrgModel(subjects);		
		addDefSubjects(subjects);
	}
	
	/**
	 * liner the constrains
	 * @param list
	 * @return
	 */
	private List<IConstrain> linerSubjects(List<IConstrain> list) {
		List<IConstrain> linerCList = new ArrayList<IConstrain>();
		Liner liner = new Liner(this.varManager);
		for(IConstrain cons : list){
			linerCList = liner.constrainLiner(cons);
		}
		return linerCList;		
	}	
	
	public ITarget getTarget(){
		return this.target;
	}
	
	public List<IConstrain> getSubjects(){
		return this.subjects;
	}
	
	public IVarManager getVarManager() {
		return this.varManager;
	}
	
	public List<IVariable> getOutputVarList() {
		return outputVars;
	}
	
	/**
	 * ܵ÷
	 * @return
	 */
	public double getScore(){
		return obj;
	}
	
	/**
	 * ΪtypeQOSԵķֵ
	 * @param type
	 * @return
	 * @throws ServiceChoicerException
	 */
	public double getScore(Type type) throws ServiceChoicerException{
		IVariable var = varManager.getVar(type,true);
		return getResult(var);
	}
	
	/**
	 * ΪtypeQOSԵֵ
	 * @param type
	 * @return
	 * @throws ServiceChoicerException
	 */
	public double getQoSAttrValue(Type type) throws ServiceChoicerException{
		IVariable var = varManager.getVar(type, false);
		double result = getResult(var);
		if(aggrTable.isProduct(type)){
			result = Math.exp(result);
		}
		return result;
	}
	
	// ѡеķб
	public List<String> getSelectedCands() throws ServiceChoicerException{
		List<String> result = new ArrayList<String>();
		for(IVariable candVar : var_CandNameMap.keySet()){
			Double d = getResult(candVar);
			if(d.intValue() == 1)
				result.add(var_CandNameMap.get(candVar));
		}
		return result;
	}
	
	// طidͶӦWSDLMAP
	public Map<String,String> getNodeWSDLMap() throws ServiceChoicerException{
		Map<String,String> result = new HashMap<String,String>();
		for(IVariable candVar : var_NodeIDMap.keySet()){
			Double d = getResult(candVar);
			ICandidate item = null;
			if(d.intValue() == 1){
				item = var_NodeIDMap.get(candVar);
				result.put(item.getServiceID(),item.getWsdlAddr());
			}
		}
		return result;
	}
	
	
	
	private double getResult(IVariable var) throws ServiceChoicerException {
		int index = -1;
		for(IVariable item : getOutputVarList()){
			if(item.equals(var)){
				index = getOutputVarList().indexOf(var);
			}
		}
		if(index >= 0 ){
			return results.get(index);
		}
		else
			throw new ServiceChoicerException();
			
	}

	/*
	 * toString() is just easy for test
	 */
	public String toSting(){
		final String end = "\n";
		String result = this.target.toString() + end;
		for(IConstrain cons : this.subjects){
			result += cons.toString() + end;
		}
		return result;
	}

	public void resolve() throws Exception{
		
		// ģ
		createTarget();
		createSubjects();
		subjects.addAll(linerSubjects(subjects));
		createOutputVars();
		
		// ݸlingo
		ForLingo lingo = new ForLingo(this);
		
		lingo.run();
		
		// 
		this.obj = lingo.getObj();
		this.results = lingo.getResults();
	}
	
	
	
}
