package org.once.serviceChoicer.lingo;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.once.serviceChoicer.ServiceChoicer;
import org.once.serviceChoicer.math.IPolynomial;
import org.once.serviceChoicer.math.IVariable;
import org.once.serviceChoicer.math.IVariable.Type;
import org.once.serviceChoicer.output.IConstrain;
import org.once.serviceChoicer.output.ITarget;
import org.once.serviceChoicer.output.IVarManager;

public class ForLingo {
	
	final static String end = ";\n";
	
	private ServiceChoicer sc;
	private File resultFile;
	private IVarManager varManager;
	
	// result
	private double obj;
	private List<Double> results;
	
	
	public ForLingo(ServiceChoicer s){
		this.sc = s;
		init();
	}
	
	private void init(){
		varManager = sc.getVarManager();
		resultFile = new File("result.txt");
		results = new ArrayList<Double>();
	}
	
	private String getTargetStr(){
		ITarget target = sc.getTarget();
		return target.getType().toString() + " = " + target.getPolynomial().toString();
	}
	
	private String getCStr (IConstrain cons){
		String result = "";
		IPolynomial pn = cons.getLeft();
		pn.unit();
		final int width = 5;
		int i = 0;
		int j = 0;
		List<String> localList = new ArrayList<String>();
		for(i = 0; i < pn.uk()/ width; i++){
			for(j = 0; j < width; j++){
				result += pn.get(i*width + j ).toString();
			}
			String name = varManager.getVar().getName();
			result += "-" + name + " = 0" + end;
			localList.add(name);
		}
		int pt = i*width;
		if(pt <= pn.uk()){
			for(String name: localList){
				result += "+"+name;
			}
			while(pt < pn.uk()){
				result += pn.get(pt).toString();
				pt++;
			}
		}
		return 	result + cons.getOp().toString() + "0";	
	}
	
	private static String getVarStr(IVariable var){
		if(var.getType()== Type.BIN)
			return "@bin(" + var.getName() + ")";
		if(var.getType() == Type.FREE)
			return "@free(" + var.getName() + ")";
		if(var.getType() == Type.INT)
			return "@int(" + var.getName() + ")";
		return null;
	}
	
	private String getVar(){
		String result = "";
		List<IVariable> vList = varManager.getVarList();
		for(int i = 0; i < vList.size(); i++){
			if(vList.get(i).getType()!= Type.DEFAULT){
				result += getVarStr(vList.get(i)) + end;
			}
		}
		return result;
	}
	
	private String getModel(){	
		
		String result = "[OBJ]" + getTargetStr() + end;
		for(IConstrain cons : sc.getSubjects()){
			result += getCStr(cons)+ end;
		}
		return result;
	}
	
	private String getData() throws Exception{
		String result = "DATA:\n";
		result += "@pointer(1) = @status()"+ end;
		result += "@pointer(2) = OBJ" + end ;
		
		// output var list
		List<IVariable> outputVars = sc.getOutputVarList();
		for(IVariable var : outputVars){
			result += "@text(" + resultFile.getAbsolutePath() + ") = " + var.getName() + end; 
		}
		result += "ENDDATA\n";
		return result;
	}
	
	private String getLingoScript(String filename) throws Exception {
		// model
		String content = "MODEL:\n" +
						getModel() + getVar() + getData() + 
						"END";
		
		// serviceChoicer.lng
		File file = new File(filename);
		System.out.println(file.getAbsolutePath());
		FileWriter fwriter;
		try {
			fwriter = new FileWriter(file);
			fwriter.write(content);
			fwriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return file.getAbsolutePath();
	}
	
	private void solveLingo(String filename) throws Exception{
		LingoSolver solver = new LingoSolver(filename);
		obj = solver.getObj();
		
		if(obj>0.00000000001) // objΪ
		{
			BufferedReader reader = new BufferedReader(new FileReader(this.resultFile));
			for(int i = 0; i < sc.getOutputVarList().size(); i++ ){
				String line = reader.readLine();
				results.add(new Double(line.trim()));
			}
			reader.close();
		}else{
			throw new IOException("can not find proper services.");
		}
//		this.resultFile.delete(); 
		
	}
	
	public void run() throws Exception{
		String lingoScript = "serviceChoicer.lng";
		String file = getLingoScript(lingoScript);
		solveLingo(file);
	}
	
	public double getObj(){
		return obj;
	}
	
	public List<Double> getResults(){
		return results;
	}

}
