package org.once.serviceChoicer.system.impl;

import org.once.serviceChoicer.exception.ServiceChoicerException;
import org.once.serviceChoicer.input.qos.IQoSAttrs;
import org.once.serviceChoicer.input.qos.IQoSAttr.Type;
import org.once.serviceChoicer.input.qos.impl.QoSAttrs;
import org.once.serviceChoicer.math.IPolynomial;
import org.once.serviceChoicer.math.impl.Polynomial;
import org.once.serviceChoicer.output.IVarManager;
import org.once.serviceChoicer.output.impl.VarManager;
import org.once.serviceChoicer.system.ICaculModle;
import org.once.serviceChoicer.system.util.AggrTable;
import org.once.serviceChoicer.system.util.IAggrTable;

public class FScore implements ICaculModle{

	private IVarManager varManager;
	private IAggrTable aggDefine;
	
	private Type type;
	private IQoSAttrs qosAttrs;
	
	public FScore(IQoSAttrs qosAttrs, IVarManager varManager, IAggrTable aggrmanager) {
		this.qosAttrs = qosAttrs;
		this.varManager = varManager;
		this.aggDefine = aggrmanager;
	}
	
	public void setType(Type type){
		this.type = type;
	}

	@Override
	public IPolynomial getPNomial() throws ServiceChoicerException {
		IPolynomial result = new Polynomial(varManager.getVar(type, false));
		double max = qosAttrs.getMAX(type);
		double min = qosAttrs.getMIN(type);
		
		if(aggDefine.isProduct(type)){
			max = Math.log(max);
			min = Math.log(min);
		}
		
//		QoSAttrDef def = new QoSAttrDef();
		if(aggDefine.isPositive(type)){
			result = result.add(new Polynomial(-1*min));
		}else
		{
			result = result.multiCoef(-1).add(new Polynomial(max));
		}
		result = result.multiCoef(1.0/(max-min));
		return result;
	}
	
	public static void main(String[] args){
		
		IQoSAttrs qosAttrs = QoSAttrs.getDefInstance();
		IVarManager varManager = new VarManager();
		
		FScore fcore = new FScore(qosAttrs, varManager,new AggrTable() );
		for(Type type: qosAttrs.getAttrTypes()){
			fcore.setType(type);
			IPolynomial pn;
			try {
				pn = fcore.getPNomial();
				System.out.println(type + ": " + pn);
			} catch (ServiceChoicerException e) {
				e.printStackTrace();
			}
		}
	}
}
